#include "bms_regist_array.h"

#include "bms_define.h"
#include "bms_util.h"
#include "bms_exception.h"

bool
BmsRegistArray::CheckConstruction( const std::string& name, const std::string& str )
{
  if ( str.length() != name.length() + 2 ) {
    return false;
  }
  std::string tmp = str.substr( 0, name.length() );
  BmsUtil::StringToUpper( tmp );
  if ( tmp.compare( name ) != 0 ) {
    return false;
  }
  if ( NOT( BmsWord::CheckConstruction( str.c_str() + name.length() ) ) ) {
    return false;
  }

  switch ( str.c_str()[name.length() + 2] ) {
  case ' ':
  case '\t':
  case '\0':
    return true;

  default:
    return false;
  }
}



BmsRegistArray::BmsRegistArray( void ) :
name_( "" ),
array_( new std::pair<bool, std::string>[BmsConst::WORD_MAX_COUNT] )
{
  for ( int i = 0; i < BmsConst::WORD_MAX_COUNT; ++i ) {
    array_[i].first = false;
  }
}

BmsRegistArray::BmsRegistArray( const std::string& name ) :
name_( name ),
array_( new std::pair<bool, std::string>[BmsConst::WORD_MAX_COUNT] )
{
  for ( int i = 0; i < BmsConst::WORD_MAX_COUNT; ++i ) {
    array_[i].first = false;
  }
}


BmsRegistArray::~BmsRegistArray()
{
  delete[] array_;
}


const std::string&
BmsRegistArray::GetName( void ) const
{
  return name_;
}

void
BmsRegistArray::SetName( const std::string& name )
{
  name_ = name;
}

unsigned int
BmsRegistArray::GetExistCount( void ) const
{
  unsigned int count = 0;
  for ( int i = 0; i < BmsConst::WORD_MAX_COUNT; ++i ) {
    if ( array_[i].first ) {
      ++count;
    }
  }
  return count;
}



bool
BmsRegistArray::IsExist( BmsWord pos ) const
{
  return array_[pos.ToInteger()].first;
}

bool
BmsRegistArray::IsNotExist( BmsWord pos ) const
{
  return NOT( this->IsExist( pos ) );
}


const std::string&
BmsRegistArray::At( BmsWord pos ) const
{
  if ( this->IsNotExist( pos ) ) {
    throw BmsOutOfRangeAccessException( typeid( *this ) );
  }
  return array_[pos.ToInteger()].second;
}

const std::string&
BmsRegistArray::operator []( BmsWord pos ) const
{
  return this->At( pos );
}


void
BmsRegistArray::Set( BmsWord pos, const std::string& value )
{
  pos.CheckValid();
  array_[pos.ToInteger()].first = true;
  array_[pos.ToInteger()].second = value;
}


void
BmsRegistArray::DeleteAt( BmsWord pos )
{
  pos.CheckValid();
  array_[pos.ToInteger()].first = false;
  array_[pos.ToInteger()].second = "";
}


std::string
BmsRegistArray::ToString( void ) const
{
  std::string tmp;
  for ( unsigned int i = 0; i < BmsConst::WORD_MAX_COUNT; ++i ) {
    if ( array_[i].first ) {
      tmp.append( "#" );
      tmp.append( name_ );
      tmp.append( BmsWord( i ).ToCharPtr() );
      tmp.append( " " );
      tmp.append( array_[i].second );
      tmp.append( "\n" );
    }
  }
  return tmp;
}
